import platform
from random import random
from geometry import Vector
from entity import Robot, Enemy, Barrier


class Game(platform.Platform):

    def load(self):
        self.fire = self.load_image("assets/fire.png", 32, 32, 16, 16)
        self.smoke = self.load_image("assets/smoke.png", 32, 32, 16, 16)
        self.background = self.load_image("assets/background.png", 2560, 720, None, None)
        self.foreground = self.load_image("assets/foreground.png", 2560, 720, None, None)
        self.robot = self.load_image("assets/robot.png", 106, 197, None, None)
        self.bullet0 = self.load_image("assets/0.png", 20, 20, 10, 10)
        self.bullet1 = self.load_image("assets/1.png", 20, 20, 10, 10)
        self.smoke = self.load_image("assets/smoke.png", 32, 32, 16, 16)
        self.fire = self.load_image("assets/fire.png", 32, 32, 16, 16)
        self.gun = self.load_image("assets/gun.png", 91, 61, 7, 30)
        self.start = self.load_image("assets/data-wipe.png", 512, 256, 256, 128)
        self.shield = self.load_image("assets/shield.png", 106, 197, 64, 256)
        self.enemy_image = self.load_image("assets/enemy.png", 128, 128, 64, 64)
        self.scores = self.load_image("assets/scores.png", 223, 97, None, None)
        self.credits = self.load_image("assets/credits.png", 1280, 400, None, None)
        self.boost_sound = self.load_sound("assets/boost.wav")
        self.death_sound = self.load_sound("assets/death.wav")
        self.hit_sound = self.load_sound("assets/hit.wav")
        self.destroy_sound = self.load_sound("assets/destroy.wav")
        self.play_sound(self.load_sound("assets/hitman.wav"), "Music", loop=True)
        self.set_channel_volume("Music", 0.5)
        self.set_channel_volume("FX", 0.5)
        self.add_slider(self.load_image("assets/music.png", 200, 50, None, None), lambda v: self.set_channel_volume("Music", v))
        self.add_slider(self.load_image("assets/fx.png", 200, 50, None, None), lambda v: self.set_channel_volume("FX", v))
        self.started = False
        self.start_pos = 0.0
        self.start_increment = 0.5
        self.completed = False
        try:
            with open("high_score.txt", "r") as f:
                self.high_score = int(f.read())
        except:
            self.high_score = 0

    def start_game(self):
        self.started = True
        self.player = Robot(self.robot, Vector(0, 197), [self.bullet0, self.bullet1], self.fire, self.smoke, self.gun)
        self.enemies = [Enemy(self.enemy_image, Vector((i+4)*500+random()*500, random()*720)) for i in range(30)]
        self.barriers = [Barrier(i*0.1+0.05, (random()-0.5)*0.05+(0.0 if i % 2 == 0 else 1.0)) for i in range(100)]
        self.score = 0.0

    def update(self, time_step):
        if self.started:
            self.camera.x = self.player.position.x - 300
            self.player.update(time_step)
            self.score += time_step
            for enemy in self.enemies:
                enemy.update(time_step)
                if enemy.position.x - self.camera.x < -128:
                    enemy.position.x += 1500 + random()*500
                    enemy.position.y = random()*720
            for barrier in self.barriers:
                barrier.update(time_step)
            self.player.handle_collisions(self.enemies,
                                          lambda: self.play_sound(self.hit_sound, "FX"),
                                          lambda: self.play_sound(self.destroy_sound, "FX"))
            if self.player.position.y < 0 - 15:
                self.player.take_damage(10)
            elif self.player.position.y > 720 - self.robot.height + 15:
                self.player.take_damage(10)
            if not self.player.alive:
                self.play_sound(self.death_sound, "FX")
                self.end_game()

    def render(self):
        self.render_image(self.background, Vector(int(self.camera.x/5120)*2560, 0), paralax=0.5)
        self.render_image(self.background, Vector(int(self.camera.x/5120)*2560+2560, 0), paralax=0.5)

        if self.started:
            for particle in self.player.exhaust:
                if particle.alive:
                    self.render_image(particle.image, particle.position)
            self.render_image(self.player.image, self.player.position)
            self.render_image(self.gun, self.player.position + self.player.gun_pivot - Vector(10, -10), rotation=self.player.aim)
            # self.display_bounding_box(self.player)
            if self.player.life > 1:
                self.render_image(self.shield, self.player.position)
            for bullet in self.player.bullets:
                if bullet.alive:
                    self.render_image(bullet.image, bullet.position)
                    # self.display_bounding_box(bullet)
            for enemy in self.enemies:
                self.render_image(self.enemy_image, enemy.position, rotation=enemy.bottom_angle)
                self.render_image(self.enemy_image, enemy.position, rotation=enemy.top_angle)
                # self.display_bounding_box(enemy)
            for barrier in self.barriers:
                self.render_image(self.enemy_image, Vector(barrier.position.x * 1280 - 64, barrier.position.y*720 - 64) + self.camera, rotation=barrier.rotation)
            self.render_image(self.foreground, Vector(int(self.camera.x/640)*2560, 0), paralax=4.0)
            self.render_image(self.foreground, Vector(int(self.camera.x/640)*2560+2560, 0), paralax=4.0)
            i = 0
            for bit in reversed(self.render_score_as_binary(self.score)):
                if bit == 0:
                    self.render_image(self.bullet1, self.camera + Vector(30+i*20, 20))
                elif bit == 1:
                    self.render_image(self.bullet0, self.camera + Vector(30+i*20, 20))
                i += 1
        else:
            self.start_pos += self.start_increment
            if self.start_pos > 15 or self.start_pos < -15:
                self.start_increment *= -1
            self.render_image(self.start, Vector(372, 450) + self.camera, rotation=self.start_pos)
            if self.completed:
                self.render_image(self.credits, Vector(0, 60) + self.camera)
        self.render_image(self.scores, Vector(20, 50) + self.camera)
        i = 0
        for bit in reversed(self.render_score_as_binary(self.high_score)):
            if bit == 0:
                self.render_image(self.bullet1, self.camera + Vector(30+i*20, 85))
            elif bit == 1:
                self.render_image(self.bullet0, self.camera + Vector(30+i*20, 85))
            i += 1

    def render_score_as_binary(self, score):
        next = score
        bits = []
        while score > 0:
            next = int(score / 2)
            if next*2 == score:
                bits.append(1)
            else:
                bits.append(0)
            score = next
        return bits[1:]

    def end_game(self):
        if self.score > self.high_score:
            self.high_score = self.score
            with open("high_score.txt", "w") as f:
                print(str(int(self.score)))
                f.write(str(int(self.score)))
        self.started = False
        self.completed = True

    def on_left_click(self, coordinates):
        if self.started:
            self.play_sound(self.boost_sound, "FX")
            self.player.velocity.y += 300.0
            self.player.velocity.x += 40.0

    def on_mouse_move(self, coordinates):
        if self.started:
            self.player.aim = (coordinates - (self.player.position + self.player.gun_pivot)).angle

    def on_key_press(self, key):
        if not self.started:
            self.start_game()


if __name__ == "__main__":
    g = Game()
    g.run()
