﻿import math
from random import random

class Vector:

    def __init__(self, x, y):
        self.x = float(x)
        self.y = float(y)

    @staticmethod
    def zero():
        return Vector(0.0, 0.0)

    def __add__(self, v):
        return Vector(self.x + v.x, self.y + v.y)

    def __sub__(self, v):
        return Vector(self.x - v.x, self.y - v.y)

    def __mul__(self, f):
        return Vector(self.x * f, self.y * f)

    def __iadd__(self, v):
        self.x += v.x
        self.y += v.y
        return self

    def __isub__(self, v):
        self.x -= v.x
        self.y -= v.y
        return self

    def __imul__(self, f):
        self.x *= f
        self.y *= f
        return self

    @property
    def length(self):
        return math.sqrt(self.y * self.y + self.x * self.x)

    @length.setter
    def length(self, length):
        if length > 0.0:
            current = self.length
            if current > 0.0:
                self *= length / current

    @property
    def angle(self):
        if self.length != 0.0:
            theta = math.degrees(math.asin(self.y / self.length))
            if self.x > 0:
                return theta
            else:
                return theta
        else:
            return 0.0

    @angle.setter
    def angle(self, theta):
        theta = math.radians(theta)
        l = self.length
        self.x, self.y = l * math.cos(theta), l * math.sin(theta)

    def normalize(self):
        self.length = 1.0

    def rotate(self, rotation):
        self.angle = (self.angle + rotation) % 360

    def random(self, amount):
        return Vector(self.x + (random() - 0.5) * amount, self.y + (random() - 0.5) * amount)

    @staticmethod
    def from_angle_and_size(angle, size):
        result = Vector(size, 0.0)
        result.angle = angle
        return result

    def between(self, point1, point2):
        min_x = point1.x if point1.x < point2.x else point2.x
        max_x = point1.x if point1.x > point2.x else point2.x
        min_y = point1.y if point1.y < point2.y else point2.y
        max_y = point1.y if point1.y > point2.y else point2.y
        if self.x > min_x and self.x < max_x:
            if self.y > min_y and self.y < max_y:
                return True
        else:
            return False


